%%  MASTER'S THESIS  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   This file is a part of a master's thesis by Johan Fredrik Holm Totland
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%	PHYSICAL PARAMETERS  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   LAB PARAMETERS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
L = 900;                % Length of pipe [m]
r = 16e-3;              % Mean radius [m]
v = 1.004e-6;           % Mean kinematic viscosity [m^2/s]
beta = 2.2e9;           % Bulk modulus [Pa]
rho = 998.2;            % Mean fluid density [kg/m^3]


%%	SYSTEM  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%	MODEL
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c0 = sqrt(beta/rho);    % Mean speed of sound [m/s]
T = L/c0;               % Time constant [s]
A0 = pi*r^2;            % Mean area [m^2]
Z0 = rho*c0/A0;         % Series impedanse
E = 8*v*T/r^2;          % Poiseuille friction coefficient
modelQP = getQPmodel(8,E,T,Z0);
[nx,nu] = size(modelQP.b);
[ny,~] = size(modelQP.c);
a = modelQP.a; b = modelQP.b; c = modelQP.c; d = modelQP.d;


%%  CONTROLLERS  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%	KALMAN FILTER
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Qn = 1;
Rn = 0.01;
Nn = 0;
[~,G,~] = kalman(modelQP,Qn,Rn,Nn);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%	LQR
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
A = [a zeros(nx,1); -c 0];
B = [b(:,2); -d(:,2)];
Q = [zeros(nx,nx) zeros(nx,1); zeros(1,nx) 1];
R = 3;
K = lqr(A,B,Q,R);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%	PID
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Kp = 2e-2;
Ki = 4e-1;
Kd = 0;








