%%  MASTER'S THESIS  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   This file is a part of a master's thesis by Johan Fredrik Holm Totland
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function SYS = getQPmodel(n,E,T,Z0)
    % CALCULATE GALERKIN COEFFICIENTS
    a = zeros(1,n);         % Gamma function coefficients (alpha)
    b = zeros(1,n);         % Window function coefficients (beta)
    e = zeros(1,n);         % Modal damping coefficient (epsilon)
    o = zeros(1,n);         % Modal natural frequency coefficient (omega)
    w = zeros(1,n);         % Attenuation factors (w)
    for i=1:n
        a(i) = (2*i-1)*pi/2;
        b(i) = (2*i-1)*pi/(2*n+1);
        o(i) = a(i)-1/4*sqrt(a(i)*E)+1/16*E;
        e(i) = 1/2*sqrt(a(i)*E)+1/8*E;
        w(i) = sin(b(i))/b(i);
    end
    b1 = 1./(2*sum(w./o.^2));
    b2 = 0;
    for i=1:n
        b2 = b2 + (-1)^(i+1)*(2*i-1)*w(i)/o(i)^2;
    end
    b2 = pi*b1*b2;
    
    % GENERATE TRANSFER FUNCTIONS
    s = tf('s');
    tf11 = 0; tf12 = 1; tf21 = 0; tf22 = 0;
    for i=1:n
        tf_i = 2*w(i)*((T*s)^2+b1*E*T*s)/((T*s)^2+e(i)*T*s+o(i)^2);
        tf11 = tf11 + Z0/(T*s)*tf_i;
        tf12 = tf12 + (-1)^i*2/((2*i-1)*pi)*tf_i;
    end

    [tf11_num, tf11_den] = tfdata(tf11,'v');
    [tf12_num, tf12_den] = tfdata(tf12,'v');
    tf_num = {tf11_num,tf12_num};
    tf_den = {tf11_den,tf12_den};
    tf_all = tf(tf_num,tf_den,'inputn', {'Q0', 'P1'}, 'outputn', {'P0'});
    SYS = ss(tf_all,'minimal');    
end