%%  MASTER'S THESIS  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   This file is a part of a master's thesis by Johan Fredrik Holm Totland
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%	PHYSICAL PARAMETERS  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   WELL BORE PARAMETERS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
L = 2000;              % Length of pipe [m]
r = 0.1;                % Mean radius [m]
v = 30e-6;              % Mean kinematic viscosity [m^2/s]
beta = 1.4e9;           % Bulk modulus [Pa]
rho = 1420;             % Mean fluid density [kg/m^3]


%%	SYSTEM  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%	GENERATE HIGH ORDER MODEL
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
m = 12;                 % Number of modes
c0 = sqrt(beta/rho);    % Mean speed of sound [m/s]
T = L/c0;               % Time constant [s]
A0 = pi*r^2;            % Mean area [m^2]
Z0 = rho*c0/A0;         % Series impedanse
E = 8*v*T/r^2;          % Poiseuille friction coefficient
v = 0;                  % Measurement noise power
modelS = getQPmodel(m,E,T,Z0);

%%  MODEL  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%	GENERATE LOW ORDER MODEL
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
n = 2;                 % Number of modes
c0 = sqrt(beta/rho);    % Mean speed of sound [m/s]
T = L/c0;               % Time constant [s]
Z0 = rho*c0/A0;         % Series impedanse
E = 8*v*T/r^2;          % Poiseuille friction coefficient
modelQP = getQPmodel(n,E,T,Z0);
[nx,nu] = size(modelQP.b); [ny,~] = size(modelQP.c);
a = modelQP.a; b = modelQP.b; c = modelQP.c; d = modelQP.d;

%%  LQG  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%	KALMAN FILTER
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Qn = 1;
Rn = 1;
Nn = 0;
[~,G,~] = kalman(modelQP,Qn,Rn,Nn);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%	LQR
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
A = [a zeros(nx,1); -c 0];
B = [b(:,2); -d(:,2)];
Q = [zeros(nx,nx) zeros(nx,1); zeros(1,nx) 1];
R = 100;
K = lqr(A,B,Q,R);

%%  PID  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%	WELL BORE
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Kp = 0.001;
Ki = 0.015;
Kd = 0;


%%  SIMULATION  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%	WELL BORE SIMULATION
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ramp = 2;       % Slope of ramp
value1 = 100;   % Pressure increase during first ramp
value2 = 50;    % Pressure increase during second ramp
t1 = 0;         % Start first ramp
t2 = 300;       % Start second ramp
t3 = 600;       % Stop simulation

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   PLOT
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figure;
controllerSwitch = -1;  % Select PI controller
sim('wellSimulation');
plot(P0_sim,'b','linewidth',2);
hold on;
controllerSwitch = 1;   % Select LQG controller
sim('wellSimulation');
plot(P0_sim,'r','linewidth',2);
plot(ref_sim,'k');
legend('P_{dh} with PI','P_{dh} with LQG','P_{dh} reference');
xlabel('Time [s]');
ylabel('Pressure [bar]');








